#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
File name: cli.py
Version: 0.1
Author: dhilipsiva <dhilipsiva@gmail.com>
Date created: 2015-08-10
"""
__author__ = "dhilipsiva"
__status__ = "development"

"""
CLI program
"""

APPKNOX = """
.______  ._______ ._______ .____/\ .______  ._______   ____   ____
:      \ : ____  |: ____  |:   /  \:      \ : .___  \  \   \_/   /
|   .   ||    :  ||    :  ||.  ___/|       || :   |  |  \___ ___/
|   :   ||   |___||   |___||     \ |   |   ||     :  |  /   _   \\
|___|   ||___|    |___|    |      \|___|   | \______/  /___/ \___\\
    |___|                  |___\  /    |___|
                                \/
"""


import logging

from click import option, echo, group, make_pass_decorator, argument, File

from appknox import AppknoxClient

logger = logging.getLogger("appknox")
logger.setLevel(10)


class Config(object):
    def __init__(self):
        self.client = None

pass_config = make_pass_decorator(Config, ensure=True)


@group()
@option('--username', envvar='APPKNOX_USERNAME', help="Username")
@option('--password', envvar='APPKNOX_PASSWORD', help="Password")
@option('--level', default=10, help="Log Level")
@pass_config
def cli(config, username, password, level):
    """
    Command line tool For Appknox's REST API
    """
    echo(APPKNOX)
    logger.setLevel(level)
    config.client = AppknoxClient(username=username, password=password)


@cli.command()
@pass_config
def validate(config):
    """
    Validate if credentials are properly configured!
    """
    echo("Your credentials are valid!")


@cli.command()
@argument('url')
@pass_config
def submit_url(config, url):
    """
    Validate if credentials are properly configured!
    """
    echo("Submitting Store URL")
    config.client.submit_url(url)


@cli.command()
@argument('file', type=File('r'))
@pass_config
def upload(config, file):
    """
    Validate if credentials are properly configured!
    """
    config.client.upload_file(file)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
File name: __init__.py
Version: 0.1
Author: dhilipsiva <dhilipsiva@gmail.com>
Date created: 2015-08-07
"""
__author__ = "dhilipsiva"
__status__ = "development"

"""
Python wrapper for Appknox's REST API
"""

import logging

import requests

from appknox.errors import MissingCredentialsError, InvalidCredentialsError, \
    ResponseError

FORMAT = '%(asctime)-15s %(message)s'
logging.basicConfig(format=FORMAT)
logger = logging.getLogger("appknox")


class AppknoxClient(object):
    """
    Appknox Client
    """

    def login(self):
        """
        Login and Get token
        """
        login_url = "%s/token/new.json" % self.api_base
        data = {
            'username': self.username,
            'password': self.password,
        }
        logger.debug('Logging In: %s', login_url)
        response = requests.post(login_url, data=data)
        json = response.json()
        if not json['success']:
            raise InvalidCredentialsError
        self.token = json['token']
        self.user_id = json['user']

    def __init__(
            self, username=None, password=None, api_key=None,
            host="beta.appknox.com", secure=True, auto_login=True):
        super(AppknoxClient, self).__init__()
        if username and password:
            self.basic_auth = True
            self.username = username
            self.password = password
        # API auth comes later
        # elif api_key:
        #     self.basic_auth = False
        #     self.api_key = api_key
        else:
            raise MissingCredentialsError
        protocol = 'http'
        if secure:
            protocol += 's'
        self.api_base = "%s://%s/api" % (protocol, host)
        self.login()

    def _request(self, req, endpoint, data):
        """
        Make a request
        """
        url = "%s/%s" % (self.api_base, endpoint)
        response = req(
            url, data=data, auth=(self.user_id, self.token))
        json = response.json()
        if response.status_code > 299 or response.status_code < 200:
            raise ResponseError(json.get("message"))
        logger.debug('Making a request: %s', url)
        return json

    def submit_url(self, store_url):
        """
        Submit a play store URL
        """
        data = {"storeURL": store_url}
        return self._request(requests.post, 'store_url', data)

    def upload_file(self, _file):
        """
        `_file` is a file-type object
        """
        data = {'content_type': 'application/octet-stream'}
        json = self._request(requests.get, 'signed_url', data)
        url = json['base_url']
        logger.info('Please wait while uploading file..: %s', url)
        requests.put(url, data=_file.read())
        data = {
            "file_key": json['file_key'],
            "file_key_signed": json['file_key_signed'],
        }
        return self._request(requests.post, 'uploaded_file', data)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
File name: errors.py
Version: 0.1
Author: dhilipsiva <dhilipsiva@gmail.com>
Date created: 2015-08-07
"""
__author__ = "dhilipsiva"
__status__ = "development"

"""
Just a list of errors
"""


class AppknoxError(Exception):

    def __init__(self, error_detail):
        super(AppknoxError, self).__init__()
        self.error_detail = error_detail

    def __str__(self):
        return "%s: %s" % (self.__class__.__name__, self.error_detail)


class MissingCredentialsError(AppknoxError):

    def __init__(self):
        super(MissingCredentialsError, self).__init__(
            "Please provide either `username/password`")


class InvalidCredentialsError(AppknoxError):

    def __init__(self):
        super(InvalidCredentialsError, self).__init__(
            "Your username / password pair is invalid."
            " Please check username & password")


class ResponseError(AppknoxError):

    def __init__(self, message):
        super(ResponseError, self).__init__(
            "Server Error Occured: %s" % message)
